/*
 * Define bank characteristics
 * bankdef.h
 * Copyright 2003 SESSAME
 * 29-Nov-2003 T.Yamasaki
 * 	Initial
 */

#define SIMULATION

#ifndef SIMULATION
#define IO_BASE_ADDRESS 0x8000     /* top of the I/O port address */
#define VRAM_BASE_ADDRESS 0x9000   /* top of the Video ram address */
#define EEPROM_PORT_ADDRESS 0xA000 /* top of the Video ram address */
#endif

typedef volatile unsigned char *Port;
typedef unsigned char Port_value;

#define TRUE 1
#define FALSE 0

/*
 * Port definition
 * All I/O ports are Active Low
 */

#define ACTIVE 0
#define INACTIVE 1

/* number of the I/O ports */
#define IOPORTS		16

/* size of Video RAM */
#define VRAMS		64

/* size of EEPROM */
#define EEPROMCELLS	1024

/* I/O port address map */
#define TIMER_COUNTER   0       /* second decrement counter */ 
#define DINGDONG        1       /* second timer */
#define SWITCH          2       /* push switch port */
#define LEVEL           3       /* battery level port */
#define LED             4       /* LED port. 0: ON, 1: OFF */
#define KEY_PORT        5       /* keyboard port */

/*
 * ASCII codes
 */
#define ASCII_BS	8    /* backspace */
#define ASCII_UP	0x11 /* DC1 */
#define ASCII_DOWN	0x12 /* DC2 */
#define ASCII_LEFT	0x13 /* DC3 */
#define ASCII_RIGHT	0x14 /* DC4 */
#define ASCII_DEL	0x7f /* Delete */
#define ASCII_TAB	'\t' /* horizontal tab */
#define ASCII_ESC	0x1b /* Escape */
#define ASCII_ENTER	0x0a /* Newline */

/*
 * Battery Level masks
 * 0: Active
 * 1: Inactive
 */
#define BATTERY0_LEVEL	0x01 /* Active: Empty */
#define BATTERY1_LEVEL	0x02 /* Active: Level1 */
#define BATTERY2_LEVEL	0x04 /* Active: Level2 */
#define BATTERY3_LEVEL	0x08 /* Active: Level3 */
#define BATTERY4_LEVEL	0x10 /* Active: Level4 */
#define BATTERY5_LEVEL	0x20 /* Active: Full */
const Port_value batteryptn[6] = {0xfe, 0xfd, 0xf9, 0xf1, 0xe1, 0xc1};

/*
 * Sleep timer related constant. Account bank auto off 
 */
#define TIME2SLEEP 60

/*
 * Switch bit masks
 * 0:make/close 1:break/open
 */
#define RESET_SWITCH	0x01	/* 0:reset / 1:nomal system reset */
#define POWER_SWITCH	0x02	/* 0:make / 1:break on/off button */
#define TIMER_SWITCH	0x04	/* 0:make / 1:break on/off button */

/*
 * LED bit masks
 * 0:Active 1:Inactive
 */
#define LEVEL_LED	0x0f
#define RESET_LED	0x10

/*
 * Signal masks
 */
#define TICK_SIGNALS	0x01 /* set every second */
/* Application must clear it */

#ifdef SIMULATION
/*
 * Shared memory information for software simulator
 */
#define SHMSZ 1	/* PAGE_SIZE should be larger than IOPORTS + VRAMS */
#define SHMID 1234 /* shared memory ID */
#endif
