/*
 * bank Hardware simulator
 * bankhw.c
 * Copyright 2003 SESSAME
 * 29-Nov-2003 T.Yamasaki
 * 	Initial
 */

#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <sys/time.h>
#include <sys/ioctl.h>
#include <stdio.h>
#include <curses.h>
#include <signal.h>
#include <string.h>
#include "bankdef.h"

Port setupshm();
char *shm;
extern void quit(void);
extern void ticktack(void);
Port ioport;
Port videoport;
Port eepromport;
int batterylevel = 0;

main()
{

  initwindows();
  /* map ports to the shared memory for I/O port simulation */
  ioport = setupshm();
  videoport = ioport + IOPORTS * sizeof(Port);
  eepromport = videoport + VRAMS * sizeof(Port);

  /* Initialize I/O port */
  ioport[TIMER_COUNTER] = 0;
  ioport[KEY_PORT] = (char) NULL ;
  ioport[LEVEL] = batteryptn[0]; /* battery level */
  ioport[SWITCH] = 0xff; /* break/open */
  ioport[LED] = 0xff; /* off */

  signal(SIGALRM, ticktack);
  alarm(1);

  while(TRUE)
    {
      /* Update user operation */
      updateops();
      /* Update windows according to input */
      updatewindows();
    }
}

Port setupshm()
{
  char c;
  int shmid;
  key_t key;
  // char *shmat();
  char *shm, *s;

  /* Name shared memory segment */
  key = SHMID;
  
  /* Create segment */
  if ((shmid = shmget(key, SHMSZ, IPC_CREAT | 0666)) < 0)
    {
      perror("shmget");
      exit(1);
    }
  
  /* Attach segment */
  if ((shm = shmat(shmid, NULL, 0)) == (char *) -1)
    {
      perror("shmat");
      exit(1);
    }

  return(shm);
}

WINDOW *panelwin, *helpwin, *eepromwin;
  
initwindows()
{
  int ysize, xsize;
  ysize = 24/2-1;
  xsize = 80/2-1;

  initscr();                    /* Always call initscr() first */
  signal(SIGINT, quit);         /* Make sure you have a 'cleanup' func */
  crmode();                     /* We want cbreak mode */
  noecho();                     /* We want to have control of chars */
  delwin(stdscr);               /* Create our own stdscr */
  stdscr = newwin(ysize, xsize, ysize, xsize);
  helpwin = newwin(ysize, xsize, 0, xsize);
  eepromwin = newwin(ysize, xsize, ysize, 0);
  panelwin = newwin(ysize, xsize, 0, 0);
  keypad(stdscr, TRUE);		/* enable keypad translation */
  /*flushok(stdscr, TRUE);*/        /* Enable flushing of stdout */
  scrollok(stdscr, TRUE);       /* Enable scrolling */
  erase();                      /* Initially, clear the screen */

  /* Window frame */
  box(panelwin, '|', '-');
  box(helpwin, '|', '-');
  box(eepromwin, '|', '-');
  
  /* Title */
  mvwaddstr(panelwin, 0, 1, "------------- Bank Panel ");
  wrefresh(panelwin);
  mvwaddstr(eepromwin, 0, 1,"-------------- EEPROM ");
  wrefresh(eepromwin);
  mvwaddstr(helpwin, 0, 1,  "------------- Operations ");
  mvwaddstr(helpwin, 1, 2,  "^c: Terminate Bank Hardware Process");
  mvwaddstr(helpwin, 2, 2,  "F6: Power make/break ");
  mvwaddstr(helpwin, 3, 2,  "F7/F8: down/up Battery Level ");
  mvwaddstr(helpwin, 4, 2,  "F9: Reset make/break ");
  mvwaddstr(helpwin, 5, 2,  "F11: Timer make/break ");
  wrefresh(helpwin);
}

updateops()
{
  int c;
  int flag = 1;
  
  ioctl(0, FIONBIO, &flag); /* set non-blocking I/O */

  c = wgetch(stdscr);
  switch(c)
    {
    case ERR: /* no input */
      break;
    case KEY_F(7):
      if (--batterylevel < 0) batterylevel = 0;
      ioport[LEVEL] = batteryptn[batterylevel];
      break;
    case KEY_F(8):
      if (++batterylevel > 5) batterylevel = 5;
      ioport[LEVEL] = batteryptn[batterylevel];
      break;
    case KEY_F(9):
      ioport[SWITCH] ^= RESET_SWITCH;break;
    case KEY_F(6):
      ioport[SWITCH] ^= POWER_SWITCH;break;
    case KEY_F(11):
      ioport[SWITCH] ^= TIMER_SWITCH;break;
    case KEY_DC: /* Delete */
      c = ASCII_DEL;break;
    case KEY_BACKSPACE:
      c = ASCII_BS;break;
    case KEY_UP: c = ASCII_UP;break;
    case KEY_DOWN: c = ASCII_DOWN;break;
    case KEY_LEFT: c = ASCII_LEFT;break;
    case KEY_RIGHT: c = ASCII_RIGHT;break;
    default:
      break;
    }
  if (((c>>8)&0xff) == 0)
    {
      while (ioport[KEY_PORT] != 0); /* wait until soft get previous key */
      ioport[KEY_PORT] = (Port_value) (c & 0xff);
    }
}

updatewindows()
{
  char c;
  int i,j;
  Port_value ptn;
  char buf[40];
  char bitstr[32];
  /*
   * Panel window
   */
  sprintf(buf,"TimerCounter[dec]: %3d", ioport[TIMER_COUNTER]);
  mvwaddstr(panelwin,1,2,buf);
  sprintf(buf,"KeyPort[hex]: %02x", ioport[KEY_PORT]);
  mvwaddstr(panelwin,2,2,buf);
  mvwaddstr(panelwin,3,2,"        RPTxxxxx");
  ptn = 1;
  for(i = 0; i < 8; i++)
    {
      bitstr[i]= ioport[SWITCH] & ptn ? '1' : '0';
      ptn <<= 1;
    }
  bitstr[8]=(char) NULL;
  sprintf(buf,"Switch: %8s", bitstr);
  mvwaddstr(panelwin,4,2,buf);
  mvwaddstr(panelwin,3,20,"     1234Rxxx");
  ptn = 1;
  for(i = 0; i < 8; i++)
    {
      bitstr[i]= ioport[LED] & ptn ? '1' : '0';
      ptn <<= 1;
    }
  bitstr[8]=(char) NULL;
  sprintf(buf,"LED: %8s", bitstr);
  mvwaddstr(panelwin,4,20,buf);

  mvwaddstr(panelwin,5,2,"       E1234Fxx");
  ptn = 1;
  for(i = 0; i < 8; i++)
    {
      bitstr[i]= ioport[LEVEL] & ptn ? '1' : '0';
      ptn <<= 1;
    }
  bitstr[8]=(char) NULL;
  sprintf(buf,"Level: %8s", bitstr);
  mvwaddstr(panelwin,6,2,buf);

  mvwaddstr(panelwin,5,20,"         txxxxxxx");
  ptn = 1;
  for(i = 0; i < 8; i++)
    {
      bitstr[i]= ioport[DINGDONG] & ptn ? '1' : '0';
      ptn <<= 1;
    }
  bitstr[8]=(char) NULL;
  sprintf(buf,"Signals: %8s", bitstr);
  mvwaddstr(panelwin,6,20,buf);

  sprintf(buf,"BatteryLevel: %d", batterylevel);
  mvwaddstr(panelwin,7,2,buf);

  /* dump video RAM */
  for (j = 0; j < 2; j++)
    {
      for (i = 0; i < 32; i++)
	{
	  c = videoport[j*32 + i];
	  buf[i] = (' ' < c) && (c < 'z'+1) ? c : ' ';
	}
      buf[32] = (char) NULL;
      mvwaddstr(panelwin,8+j,2,buf);
    }
  wrefresh(panelwin);

  /*
   * EEPROM window
   */
  /* dump EEPROM */
  for (j = 0; j < 9; j++)
    {
      for (i = 0; i < 32; i++)
	{
	  c = eepromport[j*32 + i];
	  buf[i] = (' ' < c) && (c < 'z'+1) ? c : ' ';
	}
      buf[32] = (char) NULL;
      mvwaddstr(eepromwin,j+1,2,buf);
    }
  wrefresh(eepromwin);
}

void ticktack()
{
  if (ioport[TIMER_COUNTER] != 0)
    ioport[TIMER_COUNTER]--;

  ioport[DINGDONG] |= TICK_SIGNALS;
  alarm(1);
}

void quit()
{
  erase();              /* Terminate by erasing the screen */
  refresh();
  endwin();             /* Always end with endwin() */
  delwin(curscr);       /* Return storage */
  delwin(stdscr);
  delwin(panelwin);
  delwin(helpwin);
  delwin(eepromwin);
  putchar('\n');
  exit(0);
}
