/*
 * bank software
 * banksw.c
 * Copyright 2003 SESSAME
 * 29-Nov-2003 T.Yamasaki
 *	Initial
 */
#include "bankdef.h"

#ifdef SIMULATION
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <stdio.h>
#endif

typedef int Bool;
typedef int Id;
typedef int Transition;
enum transitions {IN, KEEP, OUT};
  
Port ioport;
Port videoport;
Port eepromport;

struct Event { Id id; int value;};
enum eventlist {RESET, ONOFF, ESCAPE, ENTER, DELETE, TABRATION, OTHERKEY, LOGINOK, LOGINNG, EVENTS};
/* bank application */
Bool power = FALSE;
Port_value prev_switch, cur_switch;
Port_value prev_key, cur_key;

void printstate(Transition trans, char *string)
{
  switch (trans)
    {
    case IN:   printf("->%s\n", string); break;
    case KEEP: printf("|%s|\n", string); break;
    case OUT:  printf("%s->\n", string); break;
    }
}

void func_nopower(Transition trans, struct Event ev)
{printstate(trans, "nopower");}
void func_poweroff(Transition trans, struct Event ev)
{
  printstate(trans, "poweroff");
  switch (trans)
    {
    case IN:
      lcdclear(); break;
    case KEEP:
      /* eeprom test code */
      if (ev.id == OTHERKEY)
	  {
	    int i;
	    Port d;
	    d = eepromport;
	    for (i = 0; i < EEPROMCELLS; i++)
	      *d++ = (Port_value)(ev.value & 0x7f);
	  }
      break;
    default: break;
    }
}
void func_login(Transition trans, struct Event ev)
{
  printstate(trans, "login");
  switch (trans)
    {
    case IN:
      lcdclear();
      lcdprint("Key:");
      break;
    case KEEP:
      if (ev.id == ENTER)
	{
	  /* compare password */
	  /* no operation here */
	  
	  /* if PID is NG then */
	  ev.id = LOGINNG;
	  /* PID is always success here */
	  printf("Login ok");
	  ev.id = LOGINOK;
	  procevent(ev);
	}
      break;
    case OUT: break;
    }
}
void func_title(Transition trans, struct Event ev)
{
  printstate(trans, "title");
  switch (trans)
    {
    case IN:
      lcdclear();
      lcdprint("Title:");
      break;
    }
}
void func_address(Transition trans, struct Event ev)
{
  printstate(trans, "address");
  switch (trans)
    {
    case IN:
      lcdclear();
      lcdprint("Address:");
      break;
    }
}
void func_user(Transition trans, struct Event ev)
{
  printstate(trans, "user");
  switch (trans)
    {
    case IN:
      lcdclear();
      lcdprint("User:");
      break;
    }
}
void func_password(Transition trans, struct Event ev)
{
  printstate(trans, "password");
  switch (trans)
    {
    case IN:
      lcdclear();
      lcdprint("Password:");
      break;
    }
}
void func_confirm(Transition trans, struct Event ev)
{
  printstate(trans, "confirm");
  switch (trans)
    {
    case IN:
      lcdclear();
      lcdprint("Update?(OK:Enter/NO:Esc)");
      break;
    }
}

int main()
{
  struct Event ev;

#ifdef SIMULATION
  int shmid;
  key_t shmkey;
  char *shm;
  Port *s;

  /* Get memory segment */
  shmkey = SHMID;

  if ((shmid = shmget(shmkey, SHMSZ, 066)) < 0)
    {
      perror("shmget");
      exit(1);
    }

  /* Attach segment */
  if ((shm = shmat(shmid, NULL, 0)) == (char *) -1)
    {
      perror("shmat");
      exit(1);
    }

  ioport = (Port)shm;
  videoport = ioport + IOPORTS * sizeof(Port);
  eepromport = videoport + VRAMS * sizeof(Port);
#else
  /* real hardware connected */
  ioport = IO_BASE_ADDRESS;
  videoport = VIDEO_BASE_ADDRESS;
  eepromport = EEPROM_PORT_ADDRESS;
#endif
  
  /* initialize I/O port */
  ioport[TIMER_COUNTER] = 0;
  ioport[LED] = 0xff; 

  prev_switch = ioport[SWITCH];
  prev_key = ioport[KEY_PORT];

  while(TRUE)
    {
      /* count second */
      if ((ioport[DINGDONG] & TICK_SIGNALS) != 0)
	ioport[DINGDONG] &= ~TICK_SIGNALS;

      /*
       * event generator
       */
      if (makeevent(&ev))
	{
	  /*
	   * process event (including state transfer)
	   */
	  procevent(ev);
	}

      ioport[LED] = (ioport[LED] & ~LEVEL_LED) |
	((ioport[LEVEL]) >> 1) & LEVEL_LED;
    }

  exit(0);
}

/*
 * Generate event
 *  True: made event
 *  False: no event
 */
Bool makeevent(struct Event *pev)
{
  Bool anychange;

  anychange = FALSE;
  /* switch analysis */
  cur_switch = ioport[SWITCH];
  if (prev_switch ^ cur_switch)
    {
      /* switch status changed */
#ifdef TEST
      printf("switch: prev %02x cur %02x\n", prev_switch, cur_switch);
#endif      
      if ((~prev_switch & POWER_SWITCH) && (cur_switch & POWER_SWITCH))
	/* power make -> break */
	{
	  anychange = TRUE;
	  pev->id = ONOFF; 
	  power = ~power;
#ifdef TEST
	  printf("power = %d\n",power);
#endif
	  if (power) ioport[TIMER_COUNTER] = TIME2SLEEP;
	}
      if ((~prev_switch & RESET_SWITCH) && (cur_switch & RESET_SWITCH))
	/* reset make -> break */
	{
	  anychange = TRUE;
	  pev->id = RESET; 
	  power = FALSE;
#ifdef TEST
	  printf("power = %d\n",power);
#endif
	}
      prev_switch = cur_switch;
    }
  
  /* key analysis */
  cur_key = ioport[KEY_PORT];
  ioport[KEY_PORT] = 0; /* for hand shake */
  if (prev_key ^ cur_key)
    {
      anychange = (cur_key == 0) ? anychange:TRUE;
      pev->value = cur_key;
      switch(cur_key)
	{
	case ASCII_DEL:
	  pev->id = DELETE;break;
	case ASCII_TAB:
	  pev->id = TABRATION;break;
	case ASCII_ENTER:
	  pev->id = ENTER;break;
	case ASCII_ESC:
	  pev->id = ESCAPE;break;
	default:
	  pev->id = OTHERKEY;
	}	  
#ifdef TEST      
      printf("key: prev %02x cur %02x\n", prev_key, cur_key);
#endif      
      ioport[TIMER_COUNTER] = TIME2SLEEP;
      prev_key = cur_key;
    }
  
  if ( power )
    {
      if ( ioport[TIMER_COUNTER] == 0)
	{
	/* time to sleep */
	  pev->id = ONOFF;
	  power = FALSE;
	  anychange = TRUE;
	}
    }
  if (anychange)
    printf("event id=%d value=%d\n",pev->id, pev->value);
  return(anychange);
}

procevent(struct Event ev)
{          
  enum statelist {NOPOWER, POWEROFF, LOGIN, TITLE, ADDRESS, USER, PASSWORD, CONFIRM, STATES};
  
  /* pointer to the each state function */
  static void *state[] = {func_nopower, func_poweroff, func_login, func_title, func_address, func_user, func_password, func_confirm};
  Id state_table[STATES * EVENTS] =
    {
      POWEROFF, NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL,     NULL , /* NOPOWER */
      POWEROFF, LOGIN,    POWEROFF, POWEROFF, POWEROFF, POWEROFF, POWEROFF, NULL,     NULL , /* POWEROFF */
      POWEROFF, POWEROFF, LOGIN,    LOGIN,    LOGIN,    LOGIN,    LOGIN,    TITLE,    POWEROFF , /* LOGIN */
      POWEROFF, POWEROFF, TITLE,    CONFIRM,  CONFIRM,  ADDRESS,  TITLE,    NULL,     NULL , /* TITLE */
      POWEROFF, POWEROFF, TITLE,    CONFIRM,  CONFIRM,  USER,     ADDRESS,  NULL,     NULL , /* ADDRESS */
      POWEROFF, POWEROFF, TITLE,    CONFIRM,  CONFIRM,  PASSWORD, USER,     NULL,     NULL , /* USER */
      POWEROFF, POWEROFF, TITLE,    CONFIRM,  CONFIRM,  TITLE,    PASSWORD, NULL,     NULL , /* PASSWORD */
      POWEROFF, POWEROFF, TITLE,    TITLE,    CONFIRM,  CONFIRM,  CONFIRM,  NULL,     NULL    /* CONFIRM*/
    };
  
  static Id cur_state = NOPOWER, next_state;
  void (*pstate)(Transition, struct Event);

#ifdef TEST
  int i, j;

  printf("events  ");
  for (j=0;j<EVENTS; j++)
    printf("%03d ",j);
  printf("\n");
  for (i=0;i<STATES;i++)
    {
      printf("state %d ",i);
      for (j=0;j<EVENTS; j++)
	printf("%03d ",state_table[i * EVENTS + j]);
      printf("\n");
    }
#endif
  next_state = state_table[cur_state * EVENTS + ev.id];
  pstate = state[cur_state];
  if (cur_state == next_state)
    (*pstate)(KEEP, ev);
  else
    {
      /* exit from the state */
      (*pstate)(OUT, ev);
      cur_state = next_state;
      pstate = state[cur_state];
      /* enter to the state */
      (*pstate)(IN, ev);
    }
}

/* clear video RAM */
lcdclear()
{
  int i;
  Port d;
  d = videoport;
  for (i=0; i<64; i++) {*d++ = (Port_value)' ';};
}
/* write string to the video RAM */
lcdprint(char *s)
{
  Port d;
  d = videoport;
  while ((*d++ = (Port_value)*s++) != (Port_value)NULL);
}
